
'''Human Imitation by Nao
Author: Nehchal Jindal
Date: March 10, 2013
Email: nehchalj@gmail.com
Modified by Debidatta Dwibedi for alphabetical ordering of joints and texttospeech'''

''' This program reads human joints input (Cartesian Coordinates) generated by Kinect.
The positions are converted to Nao space. Finally, the Nao cartesian coordinates are given to Nao.'''

import config
import almath
import motion
import math
import numpy
import time
import socket
from naoqi import ALProxy

def sayWord(s):
    ttsProxy = ALProxy("ALTextToSpeech","127.0.0.1",9559)
    ttsProxy.say(s)
    return



def dist(a,b):
	dist2 = (a[0]-b[0])**2 + (a[1]-b[1])**2 + (a[2]-b[2])**2
	#dist2 = sum((a-b)**2)
	return math.sqrt(dist2)
	
def mapAndAct(motionProxy, positions)	:
	# kinectHipCenter			= positions[1:5];
	# kinectSpine 			= positions[5:9];
	# kinectShoulderCenter 	= positions[9:13];
	# kinectHead 				= positions[13:17];
	# kinectShoulderLeft 		= positions[17:21];
	# kinectElbowLeft 		= positions[21:25];
	# kinectWristLeft 		= positions[25:29];
	# kinectHandLeft 			= positions[29:33];
	# kinectShoulderRight 	= positions[33:37];
	# kinectElbowRight 		= positions[37:41];
	# kinectWristRight 		= positions[41:45];
	# kinectHandRight 		= positions[45:49];
	# kinectHipLeft 			= positions[49:53];
	# kinectKneeLeft 			= positions[53:57];
	# kinectAnkleLeft 		= positions[57:61];
	# kinectFootLeft 			= positions[61:65];
	# kinectHipRight 			= positions[65:69];
	# kinectKneeRight 		= positions[69:73];
	# kinectAnkleRight 		= positions[73:77];
	# kinectFootRight 		= positions[77:81];
	kinectAnkleLeft         = positions[1:5]
	kinectAnkleRight        = positions[5:9]
	kinectElbowLeft         = positions[9:13]
	kinectElbowRight        = positions[13:17]
	kinectFootLeft          = positions[17:21]
	kinectFootRight         = positions[21:25]
	kinectHandLeft          = positions[25:29]
	kinectHandRight         = positions[29:33]
	kinectHead              = positions[33:37]
	kinectHipCenter         = positions[37:41] 
	kinectHipLeft           = positions[41:45]
	kinectHipRight          = positions[45:49]
	kinectKneeLeft          = positions[49:53]
	kinectKneeRight         = positions[53:57]
	kinectShoulderCenter    = positions[57:61]
	kinectShoulderLeft      = positions[61:65]
	kinectShoulderRight     = positions[65:69]
	kinectSpine             = positions[69:73]
	kinectWristLeft         = positions[73:77]
	kinectWristRight        = positions[77:81]
		
	#print 'type=',type(kinectHipCenter[0])
	
	print kinectSpine
	print kinectShoulderCenter
	print kinectHipCenter
	
	print dist(kinectSpine, kinectShoulderCenter)
	print dist(kinectHipCenter, kinectShoulderCenter)
	print dist(kinectShoulderLeft, kinectElbowLeft)
	print dist(kinectShoulderLeft, kinectShoulderRight)
	print dist(kinectShoulderLeft, kinectShoulderCenter)
	
	#Convert Left and Right hands coordinates in NAO frame
	
	''' Frame conventions
	K: Kinect frame with kinect sensor as origin
	H1: Human frame with some point of human as origin
	H2: Human frame after rotation of H1
	R: Nao robot frame formed by scaling down H2'''
	
	print 'kinectHandRight', kinectHandRight
	
	#K_H1Origin = kinectSpine
	#K_H1Origin = (kinectFootLeft+kinectFootRight)/2]
	#K_H1Origin = kinectShoulderLeft
	H1_ElbowLeft = kinectElbowLeft - kinectShoulderLeft
	H1_WristLeft = kinectWristLeft - kinectShoulderLeft
	
	H1_ElbowRight = kinectElbowRight - kinectShoulderRight
	H1_WristRight = kinectWristRight - kinectShoulderRight
	#H1_HandRight = kinectHandRight - K_H1Origin
	
	#print 'H1_HandRight', H1_HandRight
	H2_ElbowLeft = numpy.array([-H1_ElbowLeft[2], -H1_ElbowLeft[0], H1_ElbowLeft[1]])
	H2_WristLeft = numpy.array([-H1_WristLeft[2], -H1_WristLeft[0], H1_WristLeft[1]]);
	#H2_HandRight = numpy.array([-H1_HandRight[2], -H1_HandRight[0], H1_HandRight[1]])
	
	H2_ElbowRight = numpy.array([-H1_ElbowRight[2], -H1_ElbowRight[0], H1_ElbowRight[1]])
	H2_WristRight = numpy.array([-H1_WristRight[2], -H1_WristRight[0], H1_WristRight[1]]);
	
	#print 'H2_HandRight', H2_HandRight
	#d1 = dist(kinectSpine, kinectShoulderCenter)
	#scaleFactor = .12650/d1
	#print 'scaleFactor=', scaleFactor
	
	#R_HandLeft = scaleFactor*H2_HandLeft
	#R_HandRight = scaleFactor*H2_HandRight
	
	
	#Calculation for LShoulderPitch and LShoulderRoll
	px = H2_ElbowLeft[0]
	py = H2_ElbowLeft[1]
	pz = H2_ElbowLeft[2]
	
	print "px, py, pz:", px, py, pz
	theta1= math.atan(-pz/px)
	if pz>0 and px<0:
		theta1 = theta1 - 3.14
	elif pz<0 and px<0:
		theta1= theta1 + 3.14
	
	if py<0:
		theta2=0;
	else:
		temp = (py)/(math.sqrt(px*px + pz*pz))
		theta2 = math.atan( abs(temp) )
	
	LShoulderPitch = theta1
	LShoulderRoll = theta2
	
	print 'LShoulderPitch, LShoulderRoll:', LShoulderPitch*(180/3.14), LShoulderRoll*(180/3.14)
	
	c1 = math.cos(theta1); c2 = math.cos(theta2); s1=math.sin(theta1); s2=math.sin(theta2);
	Rot_SL_EL = numpy.matrix([ 	[c1*c2,		-c1*s2,	s1,	px],
								[s2,	 	c2,	 	0,	py],
								[-s1*c2,	s1*s2,	c1,	pz],
								[0,			0,		0,	1]])
	Rot_SL_EL_inverse = numpy.matrix([ 	[c1*c2,		s2,		-s1*c2,	-px],
										[-c1*s2,	c2,	 	s1*s2,	-py],
										[s1,		0,		c1,		-pz],
										[0,			0,		0,		1]])		
	#Calculation for RShoulderPitch and RShoulderRoll
	px = H2_ElbowRight[0]
	py = H2_ElbowRight[1]
	pz = H2_ElbowRight[2]
	
	print "px, py, pz:", px, py, pz
	theta1= math.atan(-pz/px)
	if pz>0 and px<0:
		theta1 = theta1 - 3.14
	elif pz<0 and px<0:
		theta1= theta1 + 3.14
	
	if py>0:
		theta2=0;
	else:
		temp = (py)/(math.sqrt(px*px + pz*pz))
		theta2 = math.atan( -abs(temp) )
	
	RShoulderPitch = theta1
	RShoulderRoll = theta2
	
	print 'RShoulderPitch, RShoulderRoll:', RShoulderPitch*(180/3.14), RShoulderRoll*(180/3.14)
	
	c1 = math.cos(theta1); c2 = math.cos(theta2); s1=math.sin(theta1); s2=math.sin(theta2);
	Rot_SR_ER = numpy.matrix([ 	[c1*c2,		-c1*s2,	s1,	px],
								[s2,	 	c2,	 	0,	py],
								[-s1*c2,	s1*s2,	c1,	pz],
								[0,			0,		0,	1]])
	Rot_SR_ER_inverse = numpy.matrix([ 	[c1*c2,		s2,		-s1*c2,	-px],
										[-c1*s2,	c2,	 	s1*s2,	-py],
										[s1,		0,		c1,		-pz],
										[0,			0,		0,		1]])
	
	#Convert WristLeft joint to ElbowLeft frame	
	
	E_WristLeft = numpy.dot( Rot_SL_EL_inverse,numpy.append(H2_WristLeft, [1]) );
	E_WristLeft = numpy.asarray(E_WristLeft).reshape(-1);
	E_WristLeft = numpy.delete(E_WristLeft, 3) #Remove last 1
	print 'E_WristLeft =', E_WristLeft
	
	
	#Convert WristRight joint to ElbowRight frame	
	
	E_WristRight = numpy.dot( Rot_SR_ER_inverse,numpy.append(H2_WristRight, [1]) );
	E_WristRight = numpy.asarray(E_WristRight).reshape(-1);
	E_WristRight = numpy.delete(E_WristRight, 3) #Remove last 1
	
	
	#Calculate LElbowYaw and LElbowRoll
	px = E_WristLeft[0];
	py = E_WristLeft[1];
	pz = E_WristLeft[2];
	
	theta1 = math.atan(pz/py);
	if py>0 and pz<0:
		theta1=theta1+3.14
	elif py>0 and pz>0:
		theta1=theta1-3.14
	
	#print 'px=',px
	if px<=0:
		theta2=-3.14/2;
	else:	
		theta2 = math.atan(-abs(math.sqrt(py*py+pz*pz)/px))
	
	LElbowYaw = theta1
	LElbowRoll = theta2
	
	print 'LElbowYaw, LElbowRoll', LElbowYaw, LElbowRoll
	
	
	
	#Calculate RElbowYaw and RElbowRoll
	
	px = E_WristRight[0];
	py = E_WristRight[1];
	pz = E_WristRight[2];
	
	theta1 = math.atan(pz/py);
	if py<0 and pz<0:
		theta1=theta1-3.14
	elif py<0 and pz>0:
		theta1=theta1+3.14
	
	#print 'px=',px
	if px<=0:
		theta2=3.14/2;
	else:
		theta2 = math.atan(+abs(math.sqrt(py*py+pz*pz)/px))
	
	RElbowYaw = theta1
	RElbowRoll = theta2
	
	print 'LElbowYaw, LElbowRoll', LElbowYaw, LElbowRoll
	
	#Calculating HeadYaw and HeadRoll
	H1_Head = kinectHead-kinectShoulderCenter;
	H2_Head = numpy.array([-H1_Head[2], -H1_Head[0], H1_Head[1]])
	[px, py, pz] = H2_Head
	
	theta1 = math.atan(py/px);
	
	HeadYaw = theta1
	
	#print 'R_HandLeft', R_HandLeft
	#Give command to the NAO to reach at above calculated coordinates
	#effector = "RArm"
	#space = motion.SPACE_TORSO
	#space = motion.SPACE_NAO
	#axisMask = almath.AXIS_MASK_VEL
	#isAbsolute = True
	
	#path = list(R_HandRight) + [0,0,0]
	#print 'path', path
	#times = [0.01] #seconds
	
	#names = ["LShoulderPitch", "LShoulderRoll", "LElbowYaw", "LElbowRoll"]
	names = ["LShoulderPitch", "LShoulderRoll", "LElbowYaw", "LElbowRoll", "RShoulderPitch", "RShoulderRoll", "RElbowYaw", "RElbowRoll", "HeadYaw"]
	angleList = [LShoulderPitch, LShoulderRoll, LElbowYaw, LElbowRoll, RShoulderPitch, RShoulderRoll, RElbowYaw, RElbowRoll, HeadYaw]
	timeList = [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8]
	isAbsolute=True
	fractionMaxSpeed = 0.2
	
	time.clock();
	#motionProxy.angleInterpolation(names, angleList, timeList, isAbsolute)
	motionProxy.setAngles(names, angleList, fractionMaxSpeed)
	print 'Time surpassed=', time.clock();
	
	
	#motionProxy.positionInterpolation(effector, space, path, axisMask, times, isAbsolute)
	
	#motionProxy.

def processMsg(msg, motionProxy):
	
	line = msg
	
	if line[0] == '#':
		#The line is a comment in file
		pass
	else:
		line = line.strip(); #removing leading and trailing whitespaces and newline
		line=line.split();
		line = [float(x) for x in line]
		positions = numpy.array(line)
		#positions.append(line)
		#line = f.readline()

		print 'positions:',positions	#print positions
		
		if positions[4] != 0:   #if data is valid
			time.clock();
			mapAndAct(motionProxy, positions)
			print 'Time elapsed=', time.clock()
			#time.sleep(4)
	
	#time.sleep(4)
def main():
	
	#Create proxy to ALMotion	
	while True:		
		try:
			motionProxy = config.loadProxy("ALMotion")
			break
		except:
			print 'Error Occured, trying again...\n'
	
	
	#motionProxy = config.loadProxy("ALMotion")
	
	# Set NAO to Pose Init
	config.PoseInit (motionProxy)
	
	inputMethod = "file"
	#inputMethod = "socket"
	
	
	
	if inputMethod == "file":
		#Open file and read Kinect data	
		#f = open('kinect_data2.txt', 'r')
		#f = open('P1_1_1_p06.csv', 'r')
		#line = f.readline()
		# f = open('kinect_data.txt', 'r')

##		f = open('ds3.txt', 'r')
##		sayWord("Go upstairs")

##		f = open('ds2.txt', 'r')
##		sayWord("Go straight")

		
		f = open('ds.txt', 'r')
		sayWord("Take a right")

##		f = open('ds4.txt', 'r')
##		sayWord("Left")

		

		positions = []
		i = 0
		for line in f:
			processMsg(line, motionProxy)
			time.sleep(.033) #time in seconds
			print 'frame number =',i
			i=i+1
		f.close()
		
	
		
		
		
		
	
	
	
	
	
	

if __name__ == "__main__":
	main()
